/**
 * The outline of a ignite property which is a managed property that
 * can be used to invoke call back functions when the value of the property changes.
 */
class IgniteProperty {
    constructor() {
        this.callbacks = [];
        this._value = null;
    }

    get value() {
        return this._value;
    }

    set value(val) {
        var old = this._value;
        this._value = val;

        //Invoke any callbacks letting them know the value changed.
        for (var i = 0; i < this.callbacks.length; i++) {
            this.callbacks[i].invoke(old, val);
        }
    }

    attach(onChange) {
        var callback = new IgnitePropertyCallback(this, onChange);
        this.callbacks.push(callback);
        return callback;
    }
}

/**
 * Return the value of the property if we try to convert
 * the property to a string.
 */
IgniteProperty.prototype.toString = function () {
    return this.value.toString();
}

/**
 * The outline of a ignite property callback that allows
 * disconnecting of callbacks on demand when they are no longer needed.
 */
class IgnitePropertyCallback {
    constructor(property, callback) {
        this.callback = callback;
        this.property = property;
    }

    invoke(oldValue, newValue) {
        if (this.callback) {
            this.callback(oldValue, newValue);
        }
    }

    disconnect() {
        this.callback = null;

        if (this.property) {
            this.property.callbacks = this.property.callbacks.filter(callback => callback != this);
            this.property = null;
        }
    }
}

/**
 * The outline of a simple rendering context which allows us to 
 * know if we are currently rendering anything ignite related. This works
 * because Javascript is single threaded, if events could break the current execution
 * this would fail. But it's safe since events cannot do that.
 */
class IgniteRenderingContext {
    static enter() {
        if (!IgniteRenderingContext.RenderCount) {
            IgniteRenderingContext.RenderCount = 0;
        }

        IgniteRenderingContext.RenderCount++;
    }

    static leave() {
        if (IgniteRenderingContext.RenderCount) {
            IgniteRenderingContext.RenderCount--;
        }
    }

    static get rendering() {
        if (IgniteRenderingContext.RenderCount && IgniteRenderingContext.RenderCount > 0) {
            return true;
        }

        return false;
    }
}

window.IgniteRenderingContext = IgniteRenderingContext;

export {
    IgniteProperty
};